///license
//Christian Himpe 2010
//DCM - Dynamic Causal Modelling (dcm.cc)
///*

///includes
#include "dcm.hh"
///*

///implementation
int main(int argc, char* argv[])
{	
	checkargs(argc,argv);

	say("");
	sys("Limiting Resources");			limit(LIMIT);
	sys("Using Threads");       		maxthreads();
	sys("Importing");					Import import(argv[1],argv[2],argc);
	sys("Initializing Input and Data"); Feed feed(import.getsys(),import.getinput(),import.getdata());
	sys("Initializing System");			System system(import.getsys(),feed,import.getdyn(),import.getfor(),1);
	sys("Initializing Drift");			Drift drift(import.getsys());
	sys("Initializing Output");			Output output(import.getsys(),system,feed,drift);
	sys("Initializing Solver");			Solver solver(import.getsys(),system,drift,1);
	
	if(import.gettype()==EST)
	{
		sys("Initializing Bayes");		Bayes bayes(import.getsys(),system,solver,feed,drift);
		sys("Estimating Parameters");	bayes.expmax(); 
		sys("Writing Parameters");		output.writeTXT();
		sys("Running Solver");	        solver.solve(output);
	}
	else if(import.gettype()==SIM)
	{ 
		sys("Test Stability"); system.isstable()?ok():fail();
		sys("Running Solver"); solver.solve(output);
	}
	sys("Computing Error"); output.error();
	sys("Writing Data");	output.writeDAT();
	sys("Writing Plot");	output.writePLT();
	//sys("Writing Flow");	output.writeFLW();
	say(""); 
	
	return 0;
}
///*
