///license
//Christian Himpe 2010
//DCM - Dynamic Causal Modelling (drift.hh)
///*

///guard
#ifndef HH_DRIFT
#define HH_DRIFT
///*

///header
/* class name:	Drift
 * 
 * description:	Drift matrix provider
 * 
 * author:		Christian Himpe
 * 
 * notes:		...
 * 
 * todo:        ...
 * 
 * version:		1.0
 */
///*

///includes
#include <cmath>
#include "matrix.hh"
#include "vector.hh"
#include "structs.hh"
///*

///definitions
class Drift
{
	private:
		const syspara& sys;												//General Parameter Structure
		matrix gd;														//Global Drift Matrix
		vector b;														//Drift Parameter Vector
		int ord;														//Order of Discrete Cosine Set
		int so;															//Number of Parameters for Single State
		int eo;															//Number of Drift Parameters
		double discretecosine(int o,int n,int t) const;					//Discrete Cosine Set Member at Index t
	public:
		Drift(const syspara& s);										//Constructor
		~Drift() { ; };													//Destructor
		matrix global() const { return gd; };							//Get Global Drift Matrix
		matrix local(int t) const { return gd.copy(t*sys.states,0,sys.states,eo); };	//Get Local Drift Matrix at Index t
		vector getpara() const { return b; };							//Get 'Initial Values'
		int getparas() const { return eo; };							//Get Number of Drift Parameters
		void reset(const vector& v) { b = v; };							//Reset Parameters
};
///*

///Implementation
double Drift::discretecosine(int o,int n,int t) const
{
	return (sqrt(2.0/double(n)) * (cos(double(o)*M_PI*double(t)/double(n))));
}

Drift::Drift(const syspara& s)
 : sys(s), 
   gd(matrix(s.states*s.steps,s.states*(s.drift_order+1))),
   b(vector(s.states*(s.drift_order+1),ROW)),
   ord(s.drift_order),
   eo(s.states*(s.drift_order+1)),
   so(s.drift_order+1)
{
	if(sys.type==SIM)
	{
		for(int i=0;i<(s.drift_order+1)*sys.states;i++) { b(i,sys.dp[i]); }
	}
	
	//build global drift matrix
	for(int i=0;i<sys.steps;i++)
	{
		for(int j=0;j<sys.states;j++)
		{
			//gd(i*sys.states+j,j*so,1.0);
			for(int k=0;k<so;k++)
			{
				gd(i*sys.states+j,j*so+k,discretecosine(k,sys.steps,i));
			}
		}
	}
	//*
	
	tty("[   "); if(ord<10 && ord>=0) { tty(" "); } tty(ord); say(".    ]");
}
///*

#endif
