///license
//Christian Himpe 2010
//DCM - Dynamic Causal Modelling (dynbase.hh)
///*

///guard
#ifndef HH_DYNBASE
#define HH_DYNBASE
///*

///header
/* class name:	Dynbase
 * 
 * description:	base class for dynamic system sub model
 * 
 * author:		Christian Himpe
 * 
 * notes:		...
 * 
 * todo:        ...
 * 
 * version:		1.0
 */
///*

///declarations
class vector;
class matrix;
struct syspara;
///*

///definitions
class Dynbase
{
	protected:
		const syspara& sys;												//Reference to System Parameter Structure
		int dim;													//Dimension of Equation
		int paras;														//Number of Parameters
		vector para;													//Parameter Vector
		matrix cova;													//Covariance Matrix
	public:
		Dynbase(const syspara& s,int d,int p) : sys(s),dim(d*s.states),paras(p),para(p,ROW),cova(p,p) { };	//Constructor
		virtual ~Dynbase() { ; };										//Virtual Destructor
		virtual vector initial() = 0;									//Virtual Initial Values
		virtual vector righthand(const vector& z,const vector& u,double t) const = 0;	//Virtual Righthandside 
		virtual void reset(const vector& d) = 0;						//Virtual Reset Paramters with Given
		virtual void last(const vector& d) = 0;							//Set Previous Result as Backstep
		int getdim() const { return dim; };								//Get Dimension of Forward Equation
		int getparas() const { return paras; };							//Get Number of Parameters
		virtual vector getpara(bool t) const = 0;						//Get Parameters
		matrix getcova() const { return cova; };						//Get Covariance Matrix
		virtual bool isstable() const = 0;								//Predict Stability
};
///*

#endif
