///license
//Christian Himpe 2010
//ERP - Event Related Potentals (erp.cc)
///*

///includes
#include <string>
#include <vector>
#include <map>
#include <dirent.h>
#include <cstdlib>

#include "vector.hh"
#include "utils.hh"
///*

///declarations
typedef std::string string;
typedef std::pair<vectorarray*,int> datcol;
typedef std::map<int,datcol> junctor;

struct eeg
{
	int inputs;
	int regions;
	int steps;
	double step;
	vectorarray input;
	vectorarray region;
};
///*

///implementation
int main(int argc, char* argv[])
{
	//check for command line para
	
	//*
	
	//assemble filelist
	std::vector<string> filelist;
	DIR* d;
	struct dirent *dirp;
	if((d=opendir(argv[1]))==0) { say("Directory not found!"); return 1; }
	string curr;
	while((dirp=readdir(d))!=0) { curr = string(dirp->d_name); if( curr.compare(".")!=0 && curr.compare("..")!=0 ) { filelist.push_back(string(argv[1])+"/"+curr); } }
	closedir(d);
	//*

	//read file contents
	std::vector<eeg> data(filelist.size());
	FILE* f;
	for(int k=0;k<filelist.size();k++)
	{
		sys(filelist[k].c_str());
		if((f=fopen(filelist[k].c_str(),"r"))==0) { say("File not found!"); return 1; }
		
		//determine number of states and inputs (read first line)
		char curr;
		int index;
		int count = 0;
		junctor junction;
		data[k].regions = 0;
		data[k].inputs = 0;
		while(true)
		{
			fscanf(f,"\"%c%i\"",&curr,&index);
			switch(curr)
			{
				case 'O': junction[count] = datcol(&(data[k].region),data[k].regions);  data[k].regions++; break;
				case 'I': junction[count] = datcol(&(data[k].input),data[k].inputs); data[k].inputs++; break;
			}
			count++;
			if(fgetc(f)!=',') { break; }
		}
		//*

		//determine step size
		double diff = 0.0;
		fscanf(f,"%lf",&diff);
		fscanf(f,"%*[^\n]%*c");
		fscanf(f,"%lf",&(data[k].step));
		data[k].step -= diff;
		//*

		//determine number of steps
		data[k].steps = 1;
		while(fscanf(f,"%*[^\n]%*c")!=EOF) { data[k].steps++; }
		//*

		//read data
		rewind(f);
		fscanf(f,"%*[^\n]%*c");
		data[k].region.resize(data[k].steps+1,vector(data[k].regions,ROW));
		data[k].input.resize(data[k].steps+1,vector(data[k].inputs,ROW));
		int miss[count];
		int last[count];
		double dat = 0.0;
		char dummy;
		fpos_t pos;
		junctor::iterator jit;
		for(int i=0;i<data[k].steps;i++)
		{
			for(int j=0;j<count;j++)
			{
				//read data and check for missing values
				if(j<count-1) { miss[j] = fscanf(f,"%lf",&dat); fgetc(f); }
				else
				{
					fgetpos(f,&pos);
					dummy = fgetc(f);
					if(dummy!='\n')
					{
						fsetpos(f,&pos);		
						 miss[j] = fscanf(f,"%lf",&dat);
					}
					else { miss[j] = 0; }
				} 
				//*
				
				jit = junction.find(j);

				if(jit!=junction.end())
				{
					//interpolate for missing values
					if(last[j]==0 && i>1)
					{ 
						if(miss[j]!=0)
						{
							(*jit->second.first)[i-1](jit->second.second,0.5*((*jit->second.first)[i-2](jit->second.second)+(*jit->second.first)[i](jit->second.second)));
						}
						else
						{
							(*jit->second.first)[i](jit->second.second,0.0);
						}
					}
					if(miss[j]==0 && (i==data[i].steps-1 || i<2)) { dat = 0.0; }
					//*
					
					(*jit->second.first)[i](jit->second.second,dat);

					last[j] = miss[j];
				}
			}
		}
		//*
		
		fclose(f);
		
		ok();
	}
	//*
	
	//check for number of regions and time steps
	tty("Checking for Integrity:");
	tty(" I"); tty(data[0].inputs);
	tty(" O"); tty(data[0].regions);
	tty(" L"); tty(data[0].steps);
	tty(" S"); say(data[0].step);
	for(int i=1;i<data.size();i++)
	{
		if(data[0].inputs!=data[i].inputs) { say("Number of Inputs differs!"); return 1; }
		if(data[0].regions!=data[i].regions) { say("Number of Regions differs!"); return 1; }
		if(data[0].steps!=data[i].steps) { say("Length differs!"); return 1; }
		if(data[0].step!=data[i].step) { say("Step size differs!"); return 1; }
	}
	//*

	//compute average
	eeg erp;
	erp.inputs = data[0].inputs;
	erp.regions = data[0].regions;
	erp.steps = data[0].steps;
	erp.step = data[0].step;
	erp.input = data[0].input;
	erp.region = data[0].region;
	for(int i=0;i<erp.steps;i++)
	{
		for(int j=0;j<data.size();j++)
		{
			erp.input[i] += data[j].input[i];
			erp.region[i] += data[j].region[i];
		}
	}

	for(int i=0;i<erp.steps;i++)
	{
		erp.input[i] *= 1.0 / double(data.size());
		erp.region[i] *= 1.0 / double(data.size());
	}
	//*

	//generate erp
	string erpname(argv[1]);
	int s = erpname.rfind('/');
	if(s==erpname.length()-1) { erpname.erase(erpname.length()-1); }
	erpname += "_erp.eeu";
	say(erpname.c_str());
	FILE* g = fopen(erpname.c_str(),"w");
	fprintf(g,"\"T0\",");
	for(int j=0;j<erp.inputs;j++) { fprintf(g,"\"I%i\",",j); } 
	for(int j=0;j<erp.regions;j++) { fprintf(g,"\"O%i\"",j); if(j!=erp.regions-1) { fprintf(g,","); } }
	fprintf(g,"\n");
	for(int i=0;i<erp.steps;i++)
	{
		fprintf(g,"%f,",i*erp.step);
		for(int j=0;j<erp.inputs;j++) { fprintf(g,"%f,",erp.input[i](j)); }
		for(int j=0;j<erp.regions;j++) { fprintf(g,"%f",erp.region[i](j)); if(j!=erp.regions-1) { fprintf(g,","); } }
		fprintf(g,"\n");
	}
	fclose(g);
	//*
	
	return 0;
}
	
