///license
//Christian Himpe 2010
//DCM - Dynamic Causal Modelling (feed.hh)
///*

///guard
#ifndef HH_FEED
#define HH_FEED
///*

///header
/* class name:	Feed
 * 
 * description:	Data and Input management
 * 
 * author:		Christian Himpe
 * 
 * notes:		...
 * 
 * todo:        ...
 * 
 * version:		1.0
 */
///*

///includes
#include <cmath>
#include "vector.hh"
#include "structs.hh"
#include "utils.hh"
///*

///definitions
class Feed
{
	private:
		syspara& sys;													//General Parameter Structure
		vectorarray& dat;												//Original Data Array of Vectors
		vectorarray& inp;												//Original Input Array of Vectors
		vectorarray inq;												//Precomputed Events and Timecodes for Events
		
		inline int time(double t) const { return int(floor(t/sys.istep)); }; //Convert Time to Stepindex
	public:
		Feed(syspara& s,vectorarray& i,vectorarray& d);					//Constructor
		~Feed() { ; };													//Destructor
		vector data(int i) const { return dat.at(i); };					//Data at Step
		vector input(int i) const { return inp.at(i); };				//Input at Step 
		vector jnput(double t) const { return inq.at(time(t)); }; 		//Input at Timecode 
};
///*

///Implementation
Feed::Feed(syspara& s,vectorarray& i,vectorarray& d)
 : sys(s),
   inp(i),
   dat(d)
{
	//scale data
	if(sys.type==EST) { for(int h=0;h<sys.steps;h++) { dat[h] *= sys.feed_datascale; } }
	//*	

	if(sys.dtype==DYNLIN || sys.dtype==DYNNON)
	{
		//copy input
		inq.resize((sys.steps+1)*sys.sample,vector(sys.inputs,ROW));
		for(int h=0;h<sys.steps;h++)
		{
			for(int j=0;j<sys.sample;j++)
			{
				inq[h*sys.sample+j] = inp[h];
			}
		}
		//*
		
		//normalize input
		vector ri(sys.inputs,ROW);
		for(int h=0;h<sys.steps;h++) { ri += inq[h]; }
		ri = ri.reciprocal();
		//for(int h=0;h<sys.steps;h++) { inq[h] = inq[h].hadamard(ri); }
		//*
	}
	else if(sys.dtype==DYNEEG || sys.dtype==DYNEEE)
	{
		//determine events (and assemble peristimulus times)
		inq.resize((sys.steps+1)*sys.sample,vector(2,ROW));
		int prev = 0;
		int keep = 0;
		double tcod = 0;
		sys.events = 1;
		for(int h=0;h<sys.steps;h++)
		{
			for(int j=0;j<sys.inputs;j++)
			{
				if(i[h](j)>=0.1 && prev==0) { prev=1; keep=sys.events; sys.events++; tcod = h*sys.sample*sys.istep; }
				else if(i[h](j)<0.1 && prev==1) { prev=0; keep=0; tcod = h*sys.sample*sys.istep; }
				for(int k=0;k<sys.sample;k++)
				{
					inq[h*sys.sample+k](0,keep);
					if(sys.dtype==DYNEEG) { inq[h*sys.sample+k](1,tcod); }
					if(sys.dtype==DYNEEE) { inq[h*sys.sample+k](1,inp[h](0)); }
				}
			}
		}
		//*		
	}
	
	if(sys.dtype==DYNEEG) { tty("[   "); if(sys.dyneeg_flucsorder<10) { tty(" "); } tty(sys.dyneeg_flucsorder); say(".    ]"); } else { ok(); }
}
///*

#endif
