///license
//Christian Himpe 2010
//DCM - Dynamic Causal Modelling (foreeg.hh)
///*

///guard
#ifndef HH_FOREEG
#define HH_FOREEG
///*

///header
/* class name:	Foreeg
 * 
 * description:	class for eeg forward system sub model
 * 
 * author:		Christian Himpe
 * 
 * notes:		...
 * 
 * todo:        ...
 * 
 * version:		1.0
 */
///*

///includes
#include "vector.hh"
#include "matrix.hh"
#include "structs.hh"
#include "forbase.hh"
///*

///declarations
#define FEDIM 0															//Forward EEG Dimensions per Output
#define FEPAR 7															//Forward EEG Parameters per Output

#define FEKPM 1.0														//Forward EEG Polarity Prior Mean
#define FEKPV 1.0														//Forward EEG Polarity Prior Variance
#define FELPPM 8.0														//Forward EEG Position Prior Mean
#define FELPPV 0.0625													//Forward EEG Position Prior Variance
#define FELMPM 8.0														//Forward EEG Momentum Prior Mean
#define FELMPV 0.0625													//Forward EEG Momentum Prior Variance
///*

///definitions
class Foreeg : public Forbase
{
	private:
		vector lp;														//Position Lead Field Elements
		vector lm;														//Momentum Lead Field Elements
		double k;														//Scaling
		
		double lplmk;													//lp*lm*k
		
		inline void setup();											//use helper variables for common computations
	public:
		Foreeg(const syspara& s,int i,void* f=0);						//Constructor
		~Foreeg() { ; };												//Destructor
		vector initial() const { assert(0); };							//Initital Values
		vector righthand(double z,const vector& h) { assert(0); };		//Righthandside
		double getoutput(double z,const vector& h) const;				//Output
		void   reset(const vector& f);									//Reset Parameters with Given
};
///*

///implementation
void Foreeg::setup()
{
	lplmk = lp*lm*k;
}

Foreeg::Foreeg(const syspara& s,int i,void* f) : lp(3,ROW),lm(3,ROW), Forbase(FEDIM,FEPAR,i)
{
	foreegpara* fe = static_cast<foreegpara*>(f);
	
	if(s.type==SIM)
	{
		k=fe->k[i];
		lp(0,fe->lp[i][0]);
		lp(1,fe->lp[i][1]);
		lp(2,fe->lp[i][2]);
		lm(0,fe->lm[i][0]);
		lm(1,fe->lm[i][1]);
		lm(2,fe->lm[i][2]);
	}
	else
	{
		k=FEKPM;
		lp(0,FELPPM);
		lp(1,FELPPM);
		lp(2,FELPPM);
		lm(0,FELMPM);
		lm(1,FELMPM);
		lm(2,FELMPM);
	}
	
	para(0,k);     cova(0,0,FEKPV); 
	para(1,lp(0)); cova(1,1,FELPPV); 
	para(2,lp(1)); cova(2,2,FELPPV); 
	para(3,lp(2)); cova(3,3,FELPPV);
	para(4,lm(0)); cova(4,4,FELMPV); 
	para(5,lm(1)); cova(5,5,FELMPV); 
	para(6,lm(2)); cova(6,6,FELMPV);
	
	setup();
}

double Foreeg::getoutput(double z,const vector& h) const
{
	return lplmk*z;
}

void Foreeg::reset(const vector& f)
{
	k = f(0); para(0,k);
	lp(0,f(1)); para(1,lp(0));
	lp(1,f(2)); para(2,lp(1));
	lp(2,f(3)); para(3,lp(2));
	lm(0,f(4)); para(4,lm(0));
	lm(1,f(5)); para(5,lm(1));
	lm(2,f(6)); para(6,lm(2));
	
	setup();
}

///*

#endif 
