///license
//Christian Himpe 2010
//DCM - Dynamic Causal Modelling (forunt.hh)
///*

///guard
#ifndef HH_FORUNT
#define HH_FORUNT
///*

///header
/* class name:	Forunt
 * 
 * description:	class for eeg unit scale sub model
 * 
 * author:		Christian Himpe
 * 
 * notes:		...
 * 
 * todo:        ...
 * 
 * version:		1.0
 */
///*

///includes
#include "vector.hh"
#include "matrix.hh"
#include "structs.hh"
#include "forbase.hh"
///*

///declarations
#define FUDIM 0															//Forward Unit Dimensions per Output
#define FUPAR 1															//Forward Unit Parameters per Output

#define FUSM  1.0														//Forward Unit Scale Prior Mean
#define FUSV  0.1														//Forward Unit Scale Variance
///*

///definitions
class Forunt : public Forbase
{
	private:
		double scale;													//Output scaling
	public:
		Forunt(const syspara& s,int i,void* f=0);						//Constructor
		~Forunt() { ; };												//Destructor
		vector initial() const { assert(0); };							//Initital Values
		vector righthand(double z,const vector& h) { assert(0); };		//Righthandside
		double getoutput(double z,const vector& h) const;				//Output
		void   reset(const vector& f);									//Reset Parameters with Given
};
///*

///implementation
Forunt::Forunt(const syspara& s,int i,void* f) : Forbase(FUDIM,FUPAR,i)
{
	foruntpara* fu = static_cast<foruntpara*>(f);
	
	if(s.type==SIM) { scale = fu->s; } else { scale = FUSM; }

	para(0,FUSM);
	cova(0,0,FUSV); 
}

double Forunt::getoutput(double z,const vector& h) const
{
	return scale*z;
}

void Forunt::reset(const vector& f)
{
	para = f;
	scale = f(0);
}
///*

#endif 
