///license
//Christian Himpe 2010
//DCM - Dynamic Causal Modelling (structs.hh)
///*

///guard
#ifndef HH_STRUCTS
#define HH_STRUCTS
///*

///includes
#include <string>
///*

///declarations
typedef std::string string;

#define MRISTEP 1.0
#define EEGSTEP 0.001

#define SIM 0
#define EST 1
#define DYNLIN 0
#define DYNNON 1
#define DYNEEG 2
#define DYNEEE 3
#define FORHEM 0
#define FOREEG 1
#define FORUNT 2 

#define DRIFT_ORDER       0												//Maximum Order of Discrete Cosine Set
#define BAYES_FINITEDIFF  0.001											//Finite Difference
#define BAYES_CONVERGE    0.0001										//Convergence Accuracy
#define OUTPUT_SAMPLING   1												//Output Sampling
#define BAYES_MINLOOPS    10											//Minimum EM Runs
#define DYNEEG_INTRINSIC  128.0											//EEG Dominant Intrinsic Connection 
#define DYNEEG_FLUCSORDER 8												//Input Discrete Cosine Order
#define FEED_DATASCALE    0.001											//Data Scale
///*

///definitions
struct syspara
{
	string name;														//Basename
	string path;														//Path to File
	string inpc;														//Input Configuration
	bool type;															//Simulation (0) or Estimation (1)
	int dtype;															//Type of Dynamic System (0=Mri-Lin,1=Mri-Non,2=Eeg-I)
	int ftype;															//Type of Forward System (0=Hemo,1=Eeg,2=Unit)
	int states;															//Number of States
	int inputs;															//Number of Inputs
	int steps;															//Number Timesteps
	int events;															//Number of Events
	double step;														//Width of One Timestep
	double arterr;														//Artificial additive noise for simulations
	double istep;														//Integration Stepwidth
	int sample;															//Input Data Sampling
	int drift_order;													//Drift Order
	double* dp;															//Simulation-only drift parameters
	int dyneeg_flucsorder;												//Input Cosine Order
	int bayes_minloops;													//Minimum EM Loops
	int output_sampling;												//Output Sampling
	double dyneeg_intrinsic;											//Dyneeg Intrinsic Connection Strength
	double bayes_finitediff;											//Parameter Jacobian Finite Difference
	double bayes_converge;												//Convergence Condition 1
	double feed_datascale;												//Data Scale
};

struct dynlinpara
{
	double o;															//Sigma
	double* a;															//Connectivity Matrix Elements
	double** b;															//Input Induced Connectivity Matrices Elements
	double* c;															//Input Matrix Elements
};

struct dynnonpara
{
	double o;
	double* a;
	double** b;
	double* c;
	double* d;
};

struct dyneegpara
{
	double hi;
	double he;
	double ti;
	double te;
	double dd;
	double df;
	double t1;
	double t2;
	double* cf;
	double* cb;
	double* cl;
	double* cu;
	double* g;
	int ifo;
	double* ifp;
};

struct dyneeepara
{
	double hi;
	double he;
	double ti;
	double te;
	double ta;
	double dd;
	double df;
	double s1;
	double s2;
	double* cf;
	double* cb;
	double* cl;
	double* cu;
	double* g;
};

struct forhempara 
{
	double kappa;
	double gamma;
	double tau;
	double alpha;
	double rho;
};

struct foreegpara
{
	double* k;
	double** lp;
	double** lm;
};

struct foruntpara
{
	double s;
};
///*

#endif 
